"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SqlCache = void 0;
class SqlCache {
    constructor(options) {
        this.tableName = 'cache';
        this.deleteExpiredItems = true;
        this.defaultSetOptions = {
            ttl: 300,
        };
        if (!options) {
            throw new Error('configuration object with `client` and `databaseName` must be set when initializing SqlCache');
        }
        if (!options.client) {
            throw new Error('`client` must be set when initializing SqlCache');
        }
        if (!options.databaseName) {
            throw new Error('`databaseName` must be set when initializing SqlCache');
        }
        this.client = options.client;
        this.databaseName = options.databaseName;
        if (options.tableName) {
            this.tableName = options.tableName;
        }
        if (options.deleteExpiredItems !== undefined) {
            this.deleteExpiredItems = options.deleteExpiredItems;
        }
    }
    /**
     * Retrieve an item from the cache.
     */
    get(key) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield this.promisifyQuery(`
      SELECT
        value,
        (ttl + UNIX_TIMESTAMP(created_at)) as expiry_ts
      FROM ${this.databaseName}.${this.tableName} as cacheTable
      WHERE cacheTable.key = ?
      `, [key]);
            if (data && data.length) {
                const row = data[0];
                // Delete expired cache items
                if (this.deleteExpiredItems && Date.now() >= row.expiry_ts * 1000) {
                    yield this.delete(key);
                    return;
                }
                // Return cached item
                return row.value;
            }
        });
    }
    /**
     * Put a new item in the cache.
     */
    set(key, value, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const { ttl } = Object.assign(Object.assign({}, this.defaultSetOptions), options);
            yield this.promisifyQuery(`INSERT INTO ${this.databaseName}.${this.tableName} SET ?`, {
                key,
                value,
                ttl,
            });
        });
    }
    /**
     * Delete a cached item
     */
    delete(key) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.promisifyQuery(`DELETE FROM ${this.databaseName}.${this.tableName} as cacheTable WHERE cacheTable.key = ?`, [key]);
            return true;
        });
    }
    /**
     * Convert a mysql query to a promise.
     */
    promisifyQuery(query, data) {
        return new Promise((resolve, reject) => {
            this.client.query(query, data, (error, results) => {
                if (error) {
                    return reject(error);
                }
                resolve(results);
            });
        });
    }
}
exports.SqlCache = SqlCache;
